package cn.efunbox.manage.base.service;

import cn.efunbox.manage.base.entity.OperationLog;
import cn.efunbox.manage.base.repository.OperationLogRepository;
import cn.efunbox.manage.common.entity.page.OnePage;
import cn.efunbox.manage.common.enums.BaseOrderEnum;
import cn.efunbox.manage.common.helper.SortHelper;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Objects;

/**
 * OperationLogService
 * Created by xusq on 2017/11/22.
 */
@Slf4j
@Service
public class OperationLogService {

    @Autowired
    OperationLogRepository operationLogRepository;

    public ApiResult<OperationLog> saveOpertionLog(OperationLog operationLog){

        if (Objects.isNull(operationLog)) {
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }

        operationLog = operationLogRepository.save(operationLog);

        if (Objects.isNull(operationLog)) {
            return ApiResult.error(ApiCode.DATA_ERROR);
        }

        return ApiResult.ok(operationLog);
    }


    public ApiResult findPage(OperationLog operationLog, Integer pageSize, Integer pageNum, LinkedHashMap<String, BaseOrderEnum> sort) {
        log.debug("日志 findPage  : baseUser={}  pageSize={}  pageNum={} sortMap={}", JSON.toJSONString(operationLog), pageSize, pageNum, JSON.toJSONString(sort));
        //设置分页参数
        Long count = operationLogRepository.count(operationLog);
        OnePage page = new OnePage(count, pageNum, pageSize);
        //如果查询结果为空不用继续下面的调用了
        if (count == 0) {
            return ApiResult.ok(page);
        }
        //查询 Page的List结果
        List<OperationLog> data = operationLogRepository.find(operationLog, page.getStart(), page.getPageSize(), SortHelper.sortMap2Sort(sort));
        //set结果集
        page.setList(data);
        return ApiResult.ok(page);
    }

}
