package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.Department;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.DepartmentRepository;
import cn.efunbox.manage.base.service.DepartmentService;
import cn.efunbox.manage.base.vo.DeptTreeVO;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import cn.efunbox.manage.common.utils.SnowflakeIdUtil;
import com.sun.xml.internal.rngom.parse.host.Base;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * DepartmentServiceImpl
 * Created by wangys on 2019/03/07
 */
@Service
public class DepartmentServiceImpl implements DepartmentService {

    @Autowired
    DepartmentRepository departmentRepository;

    @Override
    public ApiResult getDeptTree(Long deptId) {
        return ApiResult.ok(recursiveTree(deptId,new ArrayList<BaseStatusEnum>(){{
            add(BaseStatusEnum.NORMAL);
            add(BaseStatusEnum.DEL);
            add(BaseStatusEnum.DISABLE);
        }}));
    }

    @Override
    public ApiResult list(Department department) {
        List<Department> departments = departmentRepository.findByStatusOrderBySortDesc(BaseStatusEnum.NORMAL);
        if(Objects.nonNull(departments)){
            return ApiResult.ok(departments);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(Department department) {
        if(department ==null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        department = departmentRepository.save(department);
        if(Objects.nonNull(department)){
            return ApiResult.ok(department);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Department department) {
        if(department ==null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        department = departmentRepository.update(department);
        if(Objects.nonNull(department)){
            return ApiResult.ok(department);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }


    @Override
    public ApiResult getDeptChild(Long deptId) {
        List<Department> departments = new ArrayList<>();
        departments = deptChild(deptId,departments,new ArrayList<BaseStatusEnum>(){{
            add(BaseStatusEnum.NORMAL);
            add(BaseStatusEnum.DEL);
            add(BaseStatusEnum.DISABLE);
        }});
        List<Long> deptIds = new ArrayList<>();
        departments.stream().forEach(department -> {
            deptIds.add(department.getPath());
        });
        Map<Long, Department> departmentMap = new HashMap<>();
        List<Department> departmentList = departmentRepository.findByIds(deptIds);
        departmentList.stream().forEach(department -> {
            departmentMap.put(department.getId(), department);
        });

        departments.stream().forEach(department -> {
            department.setFather(departmentMap.get(department.getPath()));
        });

//        departments.add(departmentRepository.find(deptId));
        return ApiResult.ok(departments);
    }

    @Override
    public ApiResult getDeptById(Long deptId) {
        if(Objects.isNull(deptId)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        Department department = departmentRepository.find(deptId);
        if(department ==null){
            return ApiResult.error(ApiCode.NOT_FOUND);
        }
        return ApiResult.ok(department);
    }

    @Override
    public ApiResult select() {
        return ApiResult.ok(departmentRepository.findSelect());
    }

    private DeptTreeVO recursiveTree(Long cid, List<BaseStatusEnum> status) {
        Department department = departmentRepository.find(cid);
        DeptTreeVO node = new DeptTreeVO();
        node.setLabel(department.getName());
        node.setId(department.getId());
        List<Department> childTreeNodes = departmentRepository.findByPathAndStatusInOrderBySortAsc(cid, status);
        for(Department child : childTreeNodes){
            DeptTreeVO n = recursiveTree(child.getId(), status); //递归
            node.getChildren().add(n);
        }
        return node;
    }

    private List<Department> deptChild(Long cid, List<Department> departments, List<BaseStatusEnum> status) {
        List<Department> childTreeNodes = departmentRepository.findByPathAndStatusInOrderBySortAsc(cid, status);
        departments.addAll(childTreeNodes);
        for(Department child : childTreeNodes){
            deptChild(child.getId(), departments, status); //递归
        }
        return departments;
    }
}
