package cn.efunbox.manage.base.controller;

import cn.efunbox.manage.base.entity.User;
import cn.efunbox.manage.base.service.UserService;
import cn.efunbox.manage.base.vo.LoginVo;
import cn.efunbox.manage.base.vo.UserVo;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import cn.efunbox.manage.common.utils.Constants;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

@RestController
@RequestMapping("/user")
public class LoginController {

    @Autowired
    private UserService userService;

    @Autowired
    private RedisTemplate<String,String> redisTemplate;

    @PostMapping("/login")
    public ApiResult<UserVo> login(@RequestBody LoginVo loginVo){
        ApiResult<UserVo> UserApiResult = userService.login(loginVo.getUserName(),loginVo.getPassword());
        if(UserApiResult.getSuccess()){
            String token = UUID.randomUUID().toString();
            UserVo UserVo = UserApiResult.getData();
            UserVo.setToken(token);
            redisTemplate.opsForValue().set(Constants.EFUNBOX_ADMIN_TOKEN_PREFIX+token, JSON.toJSONString(UserVo),30, TimeUnit.MINUTES);
        }
        return UserApiResult;
    }

    @GetMapping("/logout")
    public ApiResult<User> logout(@RequestHeader("token")String token){
        redisTemplate.delete(Constants.EFUNBOX_ADMIN_TOKEN_PREFIX+token);
        return ApiResult.ok();
    }

    @GetMapping("/tokenLogin")
    public ApiResult<Object> tokenLogin(@RequestHeader("token")String token){
        String s = redisTemplate.opsForValue().get(Constants.EFUNBOX_ADMIN_TOKEN_PREFIX + token);
        if(s==null){
            return ApiResult.error(ApiCode.INVALID_TOKEN);
        }else{
            return ApiResult.ok(JSONObject.parse(s));
        }

    }
}
